<?php

declare(strict_types=1);

namespace Qls\ServicePoints;

use Qls\ServicePoints\Service\ShippingMethodService;
use Qls\ServicePoints\Service\SystemConfigService;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Plugin;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;

class QlsServicePoints extends Plugin
{
    public function activate(ActivateContext $activateContext): void
    {
        parent::activate($activateContext);

        $context = $activateContext->getContext();

        $servicePointSystemConfig = $this->getSystemConfigService()->getServicePointSystemConfig(
            $context
        );

        if (!$servicePointSystemConfig) {
            $this->getShippingMethodService()->createServicePointShippingMethod($context);
        }
    }

    private function getSystemConfigService(): SystemConfigService
    {
        /** @var EntityRepositoryInterface $systemConfigRepository */
        $systemConfigRepository = $this->container->get('system_config.repository');

        return new SystemConfigService($systemConfigRepository);
    }

    /**
     * @return ShippingMethodService|object
     */
    private function getShippingMethodService(): ShippingMethodService
    {
        return $this->container->get(ShippingMethodService::class);
    }

    public function uninstall(UninstallContext $uninstallContext): void
    {
        if (!$uninstallContext->keepUserData()) {
            $this->getSystemConfigService()->ensureServicePointSystemConfigIsDeleted($uninstallContext->getContext());
        }

        parent::uninstall($uninstallContext);
    }
}
