<?php

declare(strict_types=1);

namespace Qls\ServicePoints\Service;

use Qls\ServicePoints\Struct\ServicePoint;
use Shopware\Core\Checkout\Customer\Aggregate\CustomerAddress\CustomerAddressEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;

class CustomerAddressService
{
    private EntityRepositoryInterface $customerAddressRepository;

    public function __construct(EntityRepositoryInterface $customerAddressRepository)
    {
        $this->customerAddressRepository = $customerAddressRepository;
    }

    public function addServicePointToCustomFields(ServicePoint $servicePoint, CustomerAddressEntity $customerAddressEntity, Context $context): void
    {
        $this->customerAddressRepository->upsert([[
            'id' => $customerAddressEntity->getId(),
            'customFields' => $servicePoint->toArray(),
        ]], $context);
    }

    public function removeServicePointFromCustomFields(CustomerAddressEntity $customerAddressEntity, Context $context): void
    {
        $this->customerAddressRepository->upsert([[
            'id' => $customerAddressEntity->getId(),
            'customFields' => null
        ]], $context);
    }

    private function getCustomerAddress(string $customerAddressEntityId, Context $context): ?CustomerAddressEntity
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('id', $customerAddressEntityId));

        return $this->customerAddressRepository->search($criteria, $context)->first();
    }

    public function ensureServicePointIsRemovedFromCustomFields(string $customerAddressEntityId, Context $context): void
    {
        $customerAddressEntity = $this->getCustomerAddress($customerAddressEntityId, $context);

        if (!$customerAddressEntity) {
            return;
        }

        if (!$customFields = $customerAddressEntity->getCustomFields()) {
            return;
        }

        if (!in_array(ServicePoint::CODE, array_keys($customFields))) {
            return;
        }

        $this->removeServicePointFromCustomFields($customerAddressEntity, $context);
    }
}
